%
%         **  Direct Adaptive Linear Equalizer Laboratory  **
%
% The DALE Lab is a matlab5-based interactive simulation tool aimed 
% at studying the convergence behavior of the LMS algorithm (and popular
% variations) when used in direct adaptive linear channel equalization. 
%
% DALE allows you to specify the channel impulse response coefficients 
% and levels of channel noise and interference for experimentation.
% It is capable of plotting the frequency response of the channel,
% the MMSE optimal equalizer, and the optimal system (chan+eq) response.
% For length-2 equalizers, it can also plot contours of the MSE surface
% in equalizer space.  In addition, it can return the eigenvalue spread 
% of the given channel and superimpose the principle axes of the 
% quadratic cost ellipsoid on an existing error surface.
%
% Given a channel, DALE can calculate the trajectory of an adaptation 
% algorithm and plot the squared-error history.  When an error surface 
% is displayed, the user can select the equalizer intialization and 
% the trajectory will be superimposed on the error surface.   
% Variants of LMS algorithm are available for experimentation, as is 
% the stepsize, the implementation delay, and the number of iterations.  
%
% After an adaptation run has completed, the frequency response
% of any equalizer along the adaptation history can be displayed.
% If a valid surface plot is present, this equalizer is selected
% from the surface.  If only an error history is present, this 
% equalizer is selected from a point along the adaptation history.
% A round marker is then placed on graph to indicate the selection.
% 
% Channel Options:
%   Chan Type : Offers a choice between preselected channel coefficients,
%               user-selectable ("custom") ones, and ".mat" file input 
%               where the coefs must be stored in a variable named "c".
%		Channel coefficients must be real-valued
%   Spacing   : T/2 Fractionally-spaced (FS) or Baud-spaced (BS) 
%   dB SNR    : signal-to-AWGN level in decibels
%   dB SIR    : signal-to-narrowband_interferer level in decibels
%   Normalize : normalizes the channel coeficient vector (-> unit norm)
%   Eql Axes  : forces the surface plot axis to be equally scaled
%
% Equalizer Algorithms:
%   LMS       : The standard LMS algorithm
%   N-LMS     : Normalized LMS
%   SE-LMS    : Signed-error LMS
%   SR-LMS    : Signed-regressor LMS
%   SS-LMS    : Signed-error, signed-regressor LMS
%   L-LMS     : LMS with leakage
%   G-LMS     : LMS with stepsize reduction at low error levels
%   MSE-GD    : MSE gradient descent (i.e. LMS average-system behavior)
%
% Algorithm Options:
%   Eq Len    : The number of coefficients in the equalizer
%   Delay     : The _deviation_ from the MMSE-optimal delay between the 
%               equalizer output and the training signal
%   Num Iter  : The number of (baud-spaced) symbols used in the adaptation
%   Stepsize  : The stepsize used in the adaptive algorithm
%   Err Smooth: The amount of smoothing performed on the error history
%               (min = no smoothing)
%
% Misc Details, for those who care...
%  - the source used for adaptation is gaussian and i.i.d.
%  - the interference is sinusoidal with frequency Fs/4
%  - the non user-selected initialization takes place at the origin 
%  - the T/2 implementation samples on "odd" outputs (assume {0,1,2,...})
%  - the "MSE bound" is based on the selected delay-offset, which is 
%    not necessarily optimal
%  - the error-trace smoothing uses a 2nd-order butterworth filter
%
%
% ----------------------------------------------------------------------
%
% DALE written by:   Phil "bert" Schniter, Cornell University, June 9 1997 

 % declare output figure numbers and clear them
 fig_surf = 1;
 fig_freq = 2;
 fig_hist = 3;
 if ishandle(fig_surf), delete(fig_surf); end; 
 if ishandle(fig_freq), delete(fig_freq); end; 
 if ishandle(fig_hist), delete(fig_hist); end;

 % allocate control window
 if exist('fig_menu'), 
   if ishandle(fig_menu), 
     delete(fig_menu); 
   end; 
 end;
 fig_menu = figure('IntegerHandle','off', 'Name',...
	'the Direct Adaptive Linear Equalizer laboratory',...
	'NumberTitle','off', 'MenuBar','none', 'Resize','off');
 pos = get(fig_menu,'Position'); 
 set(fig_menu,'Position',[pos(1)+400,pos(2)+100,400,330]);	
 clf; 

 % channel parameters
 uicontrol('Style','frame', 'Position',[140 175 245 135]);
 uicontrol('Style','text', 'Position',[150 275 100 25],'String',...
	'Channel Type');
 h_chan = uicontrol('Style','popupmenu', 'Position',[150 260 100 25],...
	'String','4-tap|6-tap|Custom|Read from file',...
	'Callback','ui_chan');
 uicontrol('Style','text', 'Position',[260 275 50 25], 'String','Spacing');
 h_fse = uicontrol('Style','popupmenu', 'Position',[260 260 50 25],...
	'String','FS|BS', 'Callback','ui_fse');
 uicontrol('Style','text', 'Position',[320 275 50 25], 'String','dB SNR');
 h_SNR = uicontrol('Style','edit', 'Position',[320 260 50 25],...
	'String','40', 'BackgroundColor',0.9*[1 1 1],...
	'Callback','ui_set_global');
 uicontrol('Style','text', 'Position',[150 230 160 25], 'String',...
	'Channel Coefficients');
 h_coefs = uicontrol('Style','edit', 'Position',[150 215 159 25],...
	'BackgroundColor',0.9*[1 1 1], 'Callback','ui_chan');
 uicontrol('Style','text', 'Position',[320 230 50 25], 'String','dB SIR');
 h_SIR = uicontrol('Style','edit', 'Position',[320 215 50 25],...
	'String','Inf', 'BackgroundColor',0.9*[1 1 1],...
	'Callback','ui_set_global');
 h_norm = uicontrol('Style','checkbox', 'Position', [150 185 120 20],...
        'String','Normalize Channel', 'Callback','ui_chan');
 h_axes = uicontrol('Style','checkbox', 'Position', [280 185 90 20],...
        'String','Equal Axes', 'Callback','ui_axes');

 % algorithm parameters
 uicontrol('Style','frame', 'Position',[140 20 245 135]);
 uicontrol('Style','text', 'Position',[150 120 50 25], 'String','Eq Len');
 h_N_f = uicontrol('Style','edit', 'Position',[150 105 50 25],...
        'String','2', 'BackgroundColor',0.9*[1 1 1], 'Callback','ui_N_f');
 uicontrol('Style','text', 'Position',[210 120 100 25], 'String',...
	'Equalizer Algorithm');
 h_alg = uicontrol('Style','popupmenu', 'Position',[210 105 100 25],...
        'String','LMS|N-LMS|SE-LMS|SR-LMS|SS-LMS|L-LMS|G-LMS|MSE-GD');
 uicontrol('Style','text', 'Position',[320 120 50 25], 'String','Delay');
 h_delay = uicontrol('Style','popupmenu', 'Position',[320 105 50 25],...
	'String','0', 'Callback','ui_delay');
 uicontrol('Style','text', 'Position',[150 75 50 25], 'String','Num Iter');
 h_num = uicontrol('Style','edit', 'Position',[150 60 50 25],...
	'String','5000', 'BackgroundColor',0.9*[1 1 1]);
 uicontrol('Style','text', 'Position',[210 75 100 25], 'String','Trace Color');
 h_col = uicontrol('Style','popupmenu', 'Position',[210 60 100 25],...
        'String','nova lox|sabotage red|orange peel|lemonade|fudge brownie|key lime|john deere grn|sky blue|cop blue|80s neon|kingpin|slush gray|johnny cash blk', 'Callback','ui_color');
 set(h_col,'Value',6); 
 uicontrol('Style','text', 'Position',[320 75 50 25], 'String','Stepsize');
 h_step = uicontrol('Style','edit', 'Position',[320 60 50 25],...
	'String','1e-2', 'BackgroundColor',0.9*[1 1 1]);
 uicontrol('Style','text', 'Position',[149 30 97 16], 'String',...
	'Error Smooth:  min');
 h_smooth = uicontrol('Style','slider', 'Position',[250 30 100 16],...
	'Min',0, 'Max',2.6, 'Value',1.3);
 uicontrol('Style','text', 'Position',[351 30 25 16], 'String',...
	'max');

 % action buttons 
 h_freq = uicontrol('Style','pushbutton', 'Position',[20 280 100 30],...
        'Callback','ui_freq', 'String','Freq Response');
 h_surf = uicontrol('Style','pushbutton', 'Position',[20 240 100 30],...
	'Callback','ui_surf', 'String','MSE Surface');
 h_eig = uicontrol('Style','pushbutton', 'Position',[20 200 100 30],...
	'Callback','ui_eig', 'String','Eigen-analysis');
 h_traj = uicontrol('Style','pushbutton', 'Position',[20 125 100 30],...
	'Callback','ui_traj', 'String','Adapt Equalizer');
 h_clr  = uicontrol('Style','pushbutton', 'Position',[20 85 100 30],...
        'Callback','ui_clr', 'String','Clear Traces');
 h_help = uicontrol('Style','pushbutton', 'Position',[20 45 100 30],...
        'Callback','helpwin DALE', 'String','Help');

 % text messages
 h_sprd = uicontrol('Style','text', 'Position',[10 175 120 15],...
        'BackgroundColor',[.8 .8 .8]);
 h_msg =  uicontrol('Style','text', 'Position',[15 20 115 15],...
        'BackgroundColor',[.8 .8 .8], 'String','click a button...');

 % intialize by interpreting ui defaults
 is_FSE = 2-get(h_fse,'Value');	% remember current setting of FSE/BSE 
 N_f = 2;			% a temporary value... 
 h_trace_c = [];		% contour trace record
 h_trace_g = [];		% eigenanalysis trace record
 h_trace_ff = [];		% ref freq-resp trace record
 h_trace_mse = [];		% mse bound trace record
 h_trace_h = []; 		% error-history trace record
 h_trace_t = []; 		% trajectory trace record
 h_trace_f = []; 		% freq-resp trace record
 ui_fse;			% handles fse vs. bse
				% ... which propagates through ui_chan.m
				% ... which propagates through ui_set_global.m
 ui_N_f;			% handles 2-tap vs. longer equalizers
				% ... and propagates through ui_set_global.m

 % dim invalid actions 
 set(h_clr,'Enable','off');

 % randomize seed
 rand('state',sum(100*clock));

